#include "PeripheralsConfig.h"

#define DCMI_DR_ADDRESS   0x50050028
#define FSMC_LCD_ADDRESS  0x60020000

void DCMI_Config(){
  DCMI_InitTypeDef DCMI_InitStructure;
  DMA_InitTypeDef  DMA_InitStructure;

  /*** Configures the DCMI to receive data from MT9D111 ***/
  DCMI_DeInit();

  /* DCMI configuration */
  DCMI_InitStructure.DCMI_CaptureMode = DCMI_CaptureMode_Continuous;
  DCMI_InitStructure.DCMI_SynchroMode = DCMI_SynchroMode_Hardware;
  DCMI_InitStructure.DCMI_PCKPolarity = DCMI_PCKPolarity_Rising;        // Read data on rising edge of PIXCLK
  DCMI_InitStructure.DCMI_VSPolarity  = DCMI_VSPolarity_Low;            // Data valid on VSYNC high
  DCMI_InitStructure.DCMI_HSPolarity  = DCMI_HSPolarity_Low;            // Data valid on HSYNC high
  DCMI_InitStructure.DCMI_CaptureRate = DCMI_CaptureRate_All_Frame;
  DCMI_InitStructure.DCMI_ExtendedDataMode = DCMI_ExtendedDataMode_8b;  // 8-bit data interface
  DCMI_Init(&DCMI_InitStructure);

  /* Enable interrupts from DCMI interface */
  /* OPCJONALNIE */
  //DCMI_ITConfig(DCMI_IT_FRAME, ENABLE); // Przerwanie generowane po odebraniu jednej ramki obrazu
  DCMI_ITConfig(DCMI_IT_VSYNC, ENABLE); // Przerwanie generowane przy przejsciu sygnalu VSYNC ze stanu aktywnego do nieaktywnego
  //DCMI_ITConfig(DCMI_IT_LINE, ENABLE);  // Przerwanie generowane po odebraniu jednego wiersza danych obrazu
  //DCMI_ITConfig(DCMI_IT_OVF, ENABLE);
  //DCMI_ITConfig(DCMI_IT_ERR, ENABLE);
  /* OPCJONALNIE */


  /*** Configures the DMA2 to transfer data from DCMI ***/

  /* DMA2 Stream1 DeInit */
  DMA_DeInit(DMA2_Stream1);
  while (DMA_GetCmdStatus(DMA2_Stream1) != DISABLE);

  /* DMA2 Stream1 configuration */
  DMA_InitStructure.DMA_Channel = DMA_Channel_1;  
  DMA_InitStructure.DMA_PeripheralBaseAddr = DCMI_DR_ADDRESS;               // Rejestr 32-bitowy
  DMA_InitStructure.DMA_Memory0BaseAddr = FSMC_LCD_ADDRESS;                 // Rejestry 16-bitowe
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;                   // DCMI ---> Buffer
  DMA_InitStructure.DMA_BufferSize = 1;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;   // 32-bit
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;       // 16-bit
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Enable;
  DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
  DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
  DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
  DMA_Init(DMA2_Stream1, &DMA_InitStructure);

  /* Enable DMA2 Stream 1 */
  DMA_Cmd(DMA2_Stream1, ENABLE);
  while (DMA_GetCmdStatus(DMA2_Stream1) != ENABLE);
}

void EXTI_Config(){
  EXTI_InitTypeDef EXTI_InitStructure;

  /* Connect EXTI LineX to TP IRQ pin */
  SYSCFG_EXTILineConfig(TP_IRQ_EXTI_port, TP_IRQ_EXTI_pin);
  /* Configure EXTI LineX */
  EXTI_InitStructure.EXTI_Line = TP_IRQ_EXTI_line;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);
}

void GPIO_Config(){
  GPIO_InitTypeDef GPIO_InitStructure;

  /****** Configures the XCLK signal used as input clock for MT9D111 camera module ******/

  /*** Connect MCO1 pin to AF0 ***/
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_MCO);

  /*** MCO1 GPIO configuration ***/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* Configure MCO1 output clock */
  RCC_MCO1Config(RCC_MCO1Source_PLLCLK, RCC_MCO1Div_3); //F_XCLK = 56MHz


  /****** Configures the DCMI GPIOs ******/

  /*** Connect DCMI pins to AF13 ***/
  /* HSYNC(PA4), PIXCLK(PA6) */
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource4, GPIO_AF_DCMI);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_DCMI);
  /* D5(PB6), VSYNC(PB7) */
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_DCMI);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_DCMI);
  /* D0..1(PC6/7) */
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource6, GPIO_AF_DCMI);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource7, GPIO_AF_DCMI);
  /* D2..4,6..7(PE0/1/4/5/6) */
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource0, GPIO_AF_DCMI);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource1, GPIO_AF_DCMI);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource4, GPIO_AF_DCMI);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource5, GPIO_AF_DCMI);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource6, GPIO_AF_DCMI);

  /*** DCMI GPIOs configuration ***/
  /* HSYNC(PA4), PIXCLK(PA6) */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_6;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  /* D5(PB6), VSYNC(PB7) */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  /* D0..1(PC6/7) */      
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
  GPIO_Init(GPIOC, &GPIO_InitStructure);
  /* D2..4,6..7(PE0/1/4/5/6) */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6;
  GPIO_Init(GPIOE, &GPIO_InitStructure);


  /****** Configures the FSMC GPIOs ******/
  /****** SRAM Data lines,  NOE and NWE configuration ******/

  /*** Connect FSMC pins to AF12 ***/
  /* D0..3,13..15, NOE(PD4), NWE(PD5), NE1(PD7), A16(PD11) */
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource0, GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource1, GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource4, GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource5, GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource7, GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource8, GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource9, GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource10, GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource11, GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource14, GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOD, GPIO_PinSource15, GPIO_AF_FSMC);
  /* D4..12 */
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource7 , GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource8 , GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource9 , GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource10 , GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource11 , GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource12 , GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource13 , GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource14 , GPIO_AF_FSMC);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource15 , GPIO_AF_FSMC);

  /*** FSMC GPIOs configuration ***/
  /* D0..3,13..15, NOE(PD4), NWE(PD5), NE1(PD7), A16(PD11) */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_8 | GPIO_Pin_9 |
                                GPIO_Pin_10 | GPIO_Pin_14 | GPIO_Pin_15 |
                                GPIO_Pin_4 |GPIO_Pin_5 | GPIO_Pin_7 | GPIO_Pin_11;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIO_InitStructure);
  /* D4..12 */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 |
                                GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 |
                                GPIO_Pin_15;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  /* LCD_RESET line configuration */
  GPIOD->MODER |= GPIO_MODER_MODER13_0;
  GPIOD->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR13_1;


  /****** Configures the I2C2 GPIOs ******/

  /*** Connect I2C1 pins to AF4 ***/
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_I2C1);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_I2C1);

  /*** I2C1 GPIOs configuration ***/  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOB, &GPIO_InitStructure);


  /****** Configures the PC13 pin as input floating ******/

	/*** PC13 configuration ***/
  GPIO_InitStructure.GPIO_Pin = TP_IRQ_pin;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(TP_IRQ_port, &GPIO_InitStructure);
}

void I2C_Config(){
  I2C_InitTypeDef  I2C_InitStructure;

  /*** Configures the I2C to communicate with MT9D111 camera module ***/

  /* I2C DeInit */
  I2C_DeInit(MT9D111_I2C);

  /* I2C configuration */
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
  I2C_InitStructure.I2C_OwnAddress1 = 0x00;
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
  I2C_InitStructure.I2C_ClockSpeed = 400000;  // 400kHz
  I2C_Init(MT9D111_I2C, &I2C_InitStructure);

  /* Enable I2C */
  I2C_Cmd(MT9D111_I2C, ENABLE);
}

void NVIC_Config(){
  NVIC_InitTypeDef NVIC_InitStructure;

  /* Wybranie grupy priorytetow */
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

  /* Enable and set DCMI Interrupts to the (0,0) priority level  */
  NVIC_InitStructure.NVIC_IRQChannel = DCMI_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* Enable and set EXTI Line13 Interrupt to the (0,4) priority level */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 4;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}

void RCC_Config(){
  RCC_ClocksTypeDef RCC_Clocks;

  /********/
  /* AHB1 */
  /********/
  /* Enable GPIOs clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB |
                         RCC_AHB1Periph_GPIOC | RCC_AHB1Periph_GPIOD |
                         RCC_AHB1Periph_GPIOE, ENABLE);
  /* Enable DMA2 clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);

  /********/
  /* AHB2 */
  /********/
  /* Enable DCMI clock */
  RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_DCMI, ENABLE);
  /* Enable SYSCFG clock */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

  /********/
  /* AHB3 */
  /********/
  /* Enable FSMC clock */
  RCC_AHB3PeriphClockCmd(RCC_AHB3Periph_FSMC, ENABLE);

  /********/
  /* APB1 */
  /********/
  /* Enable I2C1 clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);


  RCC_GetClocksFreq(&RCC_Clocks);

  /***********/
  /* SysTick */
  /***********/
  /* SysTick end of count event each 10ms */
  SysTick_Config(RCC_Clocks.HCLK_Frequency / 100);
  /* Configure SysTick clock source */
  SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);
}
