/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include "LCD.h"
#include "MT9D111.h"
#include "PeripheralsConfig.h"
//#include "board.h"
#include "usbd_cdc_core.h"
#include "usbd_usr.h"
#include "usb_conf.h"
#include "usbd_desc.h"


__ALIGN_BEGIN USB_OTG_CORE_HANDLE    USB_OTG_dev __ALIGN_END ;


#define TP_PRESSED  (uint8_t) 0x01
#define TP_RELEASED (uint8_t) 0x02


void Delay(__IO uint32_t nTime);
void TimingDelay_Decrement(void);
void waitForTouch(void);
uint8_t Is_TP_Pressed(void);
uint8_t Is_TP_Released(void);
void TP_Pressed(void);
void TP_Released(void);


__IO uint32_t TimingDelay;
unsigned short regVal;
char camera_state_info[64];
uint8_t TP_status = 0;

unsigned short BG_Color = Blue;
unsigned char current_line;
unsigned char current_screen;
unsigned char update_screen_x;

int main(void)
{
  __IO uint32_t i = 0;

  /*!< At this stage the microcontroller clock setting is already configured, 
  this is done through SystemInit() function which is called from startup
  file (startup_stm32fxxx_xx.s) before to branch to application main.
  To reconfigure the default setting of SystemInit() function, refer to
  system_stm32fxxx.c file
  */  

  /* Konfiguracja modulu USB */
  USBD_Init(&USB_OTG_dev,
            USB_OTG_FS_CORE_ID,
            &USR_desc,
            &USBD_CDC_cb,
            &USR_cb);

  RCC_Config();     /* Wlaczenie sygnalow taktujacych peryferia oraz konfiguracja SysTick */
  NVIC_Config();    /* Konfiguracja kontrolera NVIC */
  GPIO_Config();    /* Konfiguracja wyprowadzen */
  EXTI_Config();    /* Konfiguracja zewnetrznych zrodel przerwania */
  DCMI_Config();    /* Konfiguracja modulu DCMI */
  I2C_Config();     /* Konfiguracja kontrolera I2C1 */


  LCD_Init();                 // Inicjalizacja wyswietlacza
  LCD_Clear(BG_Color);        // Czyszczenie ekranu
  LCD_SetBackColor(BG_Color); // Ustawienie koloru tla
  LCD_SetTextColor(White);    // Ustawienie koloru tekstu
  LCD_SetFont(&Font8x8);      // Ustawienie czcionki 8x8 pikseli

  Delay(10);                  // Wait 100ms

  MT9D111_WriteReg(PAGE_SELECT_REG, 0x0); // Select Page 0 - Sensor Core
  if(MT9D111_ReadReg(0x00) == 0x1519){
    // Camera MT9D111 detected
    sprintf(camera_state_info, " MT9D111 not connected. ");
  }
  else{
    // Camera MT9D111 not detected
    sprintf(camera_state_info, " MT9D111 connected. ");
  }

  // Wymagana sekwencja umozliwiajaca swobodne operowanie bezposrednio na rejestrach czujnika kamery
  MT9D111_WriteReg(PAGE_SELECT_REG,0x1);                // Select Page 1 - IFP
  regVal = MT9D111_ReadReg(UC_BOOT_MODE_REG);
  MT9D111_WriteReg(UC_BOOT_MODE_REG, (regVal | 0x01));  // Reset microcontroller
  Delay(10);                                            // Wait 100 ms

  MT9D111_Init();                                       // Inicjalizacja kamery


  current_screen = 0;
  update_screen_x = 1;

  while(1){
    // Tutaj mozna umiescic dodatkowe funkcje wymagajace okresowego wykonania
    // niezaleznie od tego ktory ekran jest wyswietlany aktualnie i czy TP jest wcisniety


    // Sprawdz czy wcisnieto TP
    if( Is_TP_Pressed() == TP_PRESSED){
      TP_status = 0;              // Czysczenie flagi

      // Czekaj na odpuszczenie TP
      while( Is_TP_Released() != TP_RELEASED );
      TP_status = 0;              // Czysczenie flagi
      
      ++current_screen;           // Wybor nastepnego ekranu
      if( current_screen > 2 ){
        current_screen = 0;
      }
    }

    switch(current_screen){
      // Ekran informacyjny
      case 0:{
        if(update_screen_x & 0x1){
          update_screen_x <<= 1;

          current_line = 1;

          // Czyszczenie ekranu
          LCD_Clear(BG_Color);

          LCD_SetTextColor(White);
          LCD_DisplayStringLine(LINE(current_line++), (uint8_t *) camera_state_info);

          LCD_SetTextColor(Green);
          LCD_DisplayStringLine(LINE(current_line++), " Touch to continue...");
        }

        break;
      }
      // Podglad
      case 1:{
        if(update_screen_x & 0x2){
          update_screen_x <<= 1;

          // Ustawienie pozycji i wymiarow obszaru wyswietlacza dla obrazu z kamery
          LCD_SetDisplayWindow(239, 319, 240, 320);
          LCD_SetCursor(0,319);
          LCD_WriteRAM_Prepare();                   // Prepare to write GRAM

          MT9D111_WriteReg(PAGE_SELECT_REG, 0x0);   // Select Page 0 - Sensor Core
          MT9D111_WriteReg(SHUTTER_WIDTH_REG, 700); // Ustawienie wartosci przeslony

          DCMI_Cmd(DISABLE);                        // Wylacz DCMI
          DCMI->CR &= 0xFFFD;                       // Wybierz tryb podgladu
          DCMI_Cmd(ENABLE);                         // Wlacz DCMI
          DCMI_CaptureCmd(ENABLE);                  // Wlacz przyjmowanie danych obrazu
        }

        break;
      }
      // Wykonanie zdjecia
      case 2:{
        if(update_screen_x & 0x4){
          //update_screen_x <<= 1;
          update_screen_x = 1;

          DCMI_CaptureCmd(DISABLE);                 // Wylacz przyjmowanie danych obrazu
          DCMI_Cmd(DISABLE);                        // Wylacz DCMI

          // Ustawienie pozycji i wymiarow obszaru wyswietlacza dla obrazu z kamery
          LCD_SetDisplayWindow(239, 319, 240, 320);
          LCD_SetCursor(0,319);
          LCD_WriteRAM_Prepare();                   // Prepare to write GRAM

          MT9D111_WriteReg(PAGE_SELECT_REG, 0x0);   // Select Page 0 - Sensor Core
          MT9D111_WriteReg(SHUTTER_WIDTH_REG, 2000);// Ustawienie wartosci przeslony
          Delay(10);                                // Czekaj 100ms

          DCMI->CR |= DCMI_CaptureMode_SnapShot;    // Wybierz tryb wykonania zdjecia
          DCMI_Cmd(ENABLE);                         // Wlacz DCMI
          DCMI_CaptureCmd(ENABLE);                  // Wlacz przyjmowanie danych obrazu
          while(DCMI->CR & 0x1);                    // Czekaj na zakonczenie odbioru

          // Tutaj mozna wykonac odczyt obrazu z pamieci RAM wyswietlacza i zapisanie go na karte SD
        }

        break;
      }
      default:{break;}
    }
    Delay(10);                                      // Czekaj 100ms
  }
}

/**
  * @brief  Inserts a delay time.
  * @param  nTime: specifies the delay time length, in 10 ms.
  * @retval None
  */
void Delay(__IO uint32_t nTime){
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void){
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

void waitForTouch(void){
  while( !(Is_TP_Pressed()) );  // Wait until TP is pressed
  TP_status &= ~(TP_PRESSED);

  while( !(Is_TP_Released()) ); // Wait until TP is released
  TP_status &= ~(TP_RELEASED);
}

uint8_t Is_TP_Pressed(){
  uint16_t count = 0xAFFF;

  while(count--){
    if(!(TP_status & TP_PRESSED))
      return 0;
  }
  return TP_PRESSED;
}

uint8_t Is_TP_Released(){
  uint16_t count = 0xAFFF;

  while(count--){
    if(!(TP_status & TP_RELEASED))
      return 0;
  }
  return TP_RELEASED;
}

void TP_Pressed(){
  TP_status |= TP_PRESSED;
}

void TP_Released(){
  TP_status |= TP_RELEASED;
}
